// Colors
$black: #000;
$white: #fff;
$red: hsl(348, 86%, 61%);
$green: hsl(141, 53%, 53%);

$primary: #2e2e3a;
$secondary: #75abbc;
$accent: #f9dc5c;

$link: $secondary;
$link-focus-border: $secondary;

$danger: $red;
$success: $green;
