# Generated by Django 3.2.10 on 2021-12-15 06:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rgd', '0006_auto_20211210_1744'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='checksumfile',
            name='unique_url_collection',
        ),
        migrations.AddConstraint(
            model_name='checksumfile',
            constraint=models.UniqueConstraint(
                condition=models.Q(('type', 2)),
                fields=('collection', 'url'),
                name='unique_url_collection',
            ),
        ),
    ]
