# Generated by Django 4.0.3 on 2022-03-25 18:46

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('rgd', '0008_collection_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='checksumfile',
            name='collection',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name='%(class)ss',
                related_query_name='%(class)ss',
                to='rgd.collection',
            ),
        ),
        migrations.AlterField(
            model_name='collectionpermission',
            name='collection',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to='rgd.collection'
            ),
        ),
        migrations.AlterField(
            model_name='collectionpermission',
            name='user',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
            ),
        ),
        migrations.AlterField(
            model_name='spatialasset',
            name='files',
            field=models.ManyToManyField(related_name='+', to='rgd.checksumfile'),
        ),
    ]
