# Generated by Django 3.2.5 on 2021-08-12 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rgd', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='spatialasset',
            name='files',
            field=models.ManyToManyField(
                related_name='_rgd_spatialasset_files_+', to='rgd.ChecksumFile'
            ),
        ),
    ]
