# Generated by Django 3.2.11 on 2022-02-01 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rgd', '0007_auto_20211215_0608'),
    ]

    operations = [
        migrations.AddField(
            model_name='collection',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]
