{% block navlinks %}
  <nav class="navbar" role="navigation" aria-label="main navigation">
    <div class="navbar-brand">
      <a class="navbar-item logo" href="{% url 'rgd-index' %}">
        <strong>Resonant GeoData</strong>
      </a>

      <a role="button" class="navbar-burger burger" aria-label="menu" aria-expanded="false" data-target="navbarBasicExample">
        <span aria-hidden="true"></span>
        <span aria-hidden="true"></span>
        <span aria-hidden="true"></span>
      </a>
    </div>

    <div id="navbarBasicExample" class="navbar-menu">
      <a href="{% url 'statistics' %}" class="navbar-item">
        Statistics
      </a>
      <div class="navbar-end user-menu">
        {% if user.is_authenticated %}
          <div class="navbar-item">{{ request.user.email }}</div>
          {% if user.is_superuser %}
            <a href="{% url 'admin:index' %}" class="navbar-item admin">
              <i class="material-icons mr-1">
                admin_panel_settings
              </i>
              Admin
            </a>
          {% endif %}
          {% comment %}
          {% if user.is_staff %}
            <a href="{% url 'staff-dashboard' %}" class="navbar-item dashboard">
              <i class="material-icons mr-1">
                dashboard
              </i>
              Dashboard
            </a>
          {% endif %}
          {% endcomment %}
          <a href="{% url 'account_logout' %}" class="navbar-item logout">
            <i class="material-icons mr-1">
              meeting_room
            </i>
            Logout
          </a>
        {% else %}
          <a href="{% url 'account_login' %}" class="navbar-item login">
            <i class="material-icons mr-1">
              login
            </i>
            Login
          </a>
          <a href="{% url 'account_signup' %}" class="navbar-item signup">
            <i class="material-icons mr-1">
              create
            </i>
            Sign up
          </a>
        {% endif %}
      </div>
    </div>
  </nav>
{% endblock %}
